/******************************************************************************
*                            Steffen Hofmann                                  *
*                            mcu@shotech.de                                   *
*******************************************************************************
*  $Source:  $
*-----------------------------------------------------------------------------*
*  Datum der 2. Version  :  08.02.2017 19:19:10
*  Geraete Bezeichnung   :  RFM69CW-RF-Modul
*-----------------------------------------------------------------------------*
*- $Id:  $
*-----------------------------------------------------------------------------*
*
*	Headerfile
*	----------
*	Fr Hardware Festlegungen des Projekts RFM69CW-RF-Modul
* 
*
******************************************************************************/
#ifndef HW_H
#define HW_H

#include "stm32l0xx.h"
#include "stm32l0xx_hal_conf.h"
#include "stdbool.h"
#include "types.h"
#include <stdio.h>

//- Belegung der Prozessorports Port A ----------------------------------------
//         Port-Name      Funktion       Init                      Comments
// PA0     DIO2           port           I/O                       RFM39 port 
// PA1     CURRENT_RF     ADC_IN1        input                     Current RF 
// PA2     TX_2           USART2_TX      output                    RS232-2 
// PA3     RX_2           USART2_RX      input                     RS232-2     
// PA4     DIO0           port           I/O                       RFM69 port
// PA5     SCK            SPI1-SCK       output                    RFM69 port
// PA6     MISO           SPI1-MISO      input                     RFM69 port
// PA7     MOSI           SPI1-MOSI      output                    RFM69 port
// PA8     BUTTON         port           input                     RFM69 port
// PA9     frei-extern    port           input                     frei-extern
// PA10    frei-extern    port           input                     frei-extern
// PA11    frei-extern    port           input                     frei-extern ( Buchse X3 )
// PA12    frei-extern    port           input                     frei-extern ( Buchse X3 )
// PA13    SWDIO          Debugger       output                    Debugger    ( Buchse X3 )
// PA14    SWCLK          Debugger       output                    Debugger    ( Buchse X3 )
// PA15    LED_GELB       port           input                     RFM69 port  ( Buchse X3 )

//- Belegung der Prozessorports Port B ----------------------------------------
//         Port-Name      Funktion       Init                      Commentss
// PB0     LED_RED        port           output                    RFM69 port
// PB1     CS             port           output                    RFM69 port
// PB2     LED_GREEN      port           input                     RFM69 port
// PB3     DIO3           port           I/O                       RFM69 port
// PB4     RST_RFM69      port           input                     RFM69 port
// PB5     DIO5           port           I/O                       RFM69 port
// PB6     SCL            I2C1_SCL       input                     I2C Sensor     
// PB7     SDA            I2C1_SDA       bidir                     I2C Sensor    
// PB8     DIO1           port           I/O                       RFM69 port

//- Belegung der Prozessorports Port C ----------------------------------------
//         Port-Name      Funktion       Init                      Comments
//-----------------------------------------------------------------------------


// PC14    OSC32_IN       OSC32          input                     Quarz 32KHz      
// PC15    OSC32_OUT      OSC32          output                    Quarz 32KHz  


//- Belegung der DMA-Channels  ------------------------------------------------
//



//- Timer ---------------------------------------------------------------------
//


//- Interrupt Prio -----------------------------------------------------------

#define RFM69_PRE_EMPTION_PRIORITY_GROUP0           0   // Group 0 max. 4 Subs

#define RFM69_PRE_EMPTION_PRIORITY_0                0   // hchste Prio
#define RFM69_PRE_EMPTION_PRIORITY_1                1   //
#define RFM69_PRE_EMPTION_PRIORITY_2                2   //
#define RFM69_PRE_EMPTION_PRIORITY_3                3   //

#define RF_INT_PRE_EMPTION_PRIORITY                (u8)RFM69_PRE_EMPTION_PRIORITY_0

#define TIMER_PRE_EMPTION_PRIORITY                 (u8)RFM69_PRE_EMPTION_PRIORITY_3



/** @defgroup GPIO_pins_define Pin definition
  * @{
  */

//#define GPIO_PIN_0                 ((uint16_t)0x0001)  /* Pin 0 selected    */
//#define GPIO_PIN_1                 ((uint16_t)0x0002)  /* Pin 1 selected    */
//#define GPIO_PIN_2                 ((uint16_t)0x0004)  /* Pin 2 selected    */
//#define GPIO_PIN_3                 ((uint16_t)0x0008)  /* Pin 3 selected    */
//#define GPIO_PIN_4                 ((uint16_t)0x0010)  /* Pin 4 selected    */
//#define GPIO_PIN_5                 ((uint16_t)0x0020)  /* Pin 5 selected    */
//#define GPIO_PIN_6                 ((uint16_t)0x0040)  /* Pin 6 selected    */
//#define GPIO_PIN_7                 ((uint16_t)0x0080)  /* Pin 7 selected    */
//#define GPIO_PIN_8                 ((uint16_t)0x0100)  /* Pin 8 selected    */
//#define GPIO_PIN_9                 ((uint16_t)0x0200)  /* Pin 9 selected    */
//#define GPIO_PIN_10                ((uint16_t)0x0400)  /* Pin 10 selected   */
//#define GPIO_PIN_11                ((uint16_t)0x0800)  /* Pin 11 selected   */
//#define GPIO_PIN_12                ((uint16_t)0x1000)  /* Pin 12 selected   */
//#define GPIO_PIN_13                ((uint16_t)0x2000)  /* Pin 13 selected   */
//#define GPIO_PIN_14                ((uint16_t)0x4000)  /* Pin 14 selected   */
//#define GPIO_PIN_15                ((uint16_t)0x8000)  /* Pin 15 selected   */
//#define GPIO_PIN_All               ((uint16_t)0xFFFF)  /* All pins selected */

//- Pins ----------------------------------------------------------------------

#define A_DIO2              GPIO_PIN_0   //- IO2 RFM69
#define A_CURRENT_RF        GPIO_PIN_1   //- Current RFM69 Modul
#define A_TX_2              GPIO_PIN_2   //- TX-2        
#define A_RX_2              GPIO_PIN_3   //- RX-2        
#define A_DIO0              GPIO_PIN_4   //- IO0 
#define A_SCK               GPIO_PIN_5   //- RF-Modul
#define A_MISO              GPIO_PIN_6   //- RF-Modul
#define A_MOSI              GPIO_PIN_7   //- RF-Modul
#define A_BUTTON            GPIO_PIN_8   //- frei
#define A_TX1               GPIO_PIN_9   //- frei
#define A_RX1               GPIO_PIN_10  //- frei
#define A_FREI_11           GPIO_PIN_11  //- LED Red
#define A_FREI_12           GPIO_PIN_12  //- LED Green
#define A_SWDIO             GPIO_PIN_13  //- Debugger  
#define A_SWCLK             GPIO_PIN_14  //- Debugger   
#define A_LED_GELB          GPIO_PIN_15  //- frei


#define B_LED_RED           GPIO_PIN_0   //- RF-Modul
#define B_CS                GPIO_PIN_1   //- RF-Modul
#define B_LED_GREEN         GPIO_PIN_2   //- RF-Modul
#define B_DIO3              GPIO_PIN_3   //- RF-Modul
#define B_RST_RFM69         GPIO_PIN_4   //- RF-Modul
#define B_DIO5              GPIO_PIN_5   //- RF-Modul
#define B_SCL               GPIO_PIN_6   //- I2C extern
#define B_SDA               GPIO_PIN_7   //- I2C extern
#define B_DIO1              GPIO_PIN_8   //- RF-Modul

#define C_OSC32_IN          GPIO_PIN_14  //- Quarz 32KHz
#define C_OSC32_OUT         GPIO_PIN_15  //- Quarz 32KHz

//-----------------------------------------------------------------------------
//-  Pin Define
//-----------------------------------------------------------------------------
//#define PIN_DIO2              0   //- IO2 RFM69
//#define PIN_CURRENT_RF        1   //- Current RFM69 Modul
//#define PIN_TX2               2   //- TX-2        
//#define PIN_RX_2              3   //- RX-2        
//#define PIN_DIO0              4   //- IO0 
//#define PIN_SCK               5   //- RF-Modul
//#define PIN_MISO              6   //- RF-Modul
//#define PIN_MOSI              7   //- RF-Modul
//#define PIN_TX1               9
//#define PIN_RX1               10
//#define PIN_LED_RED           11  //- LED Red
//#define PIN_LED_GREEN         12  //- LED Green
//
//
//#define PIN_CS                1   //- RF-Modul
//#define PIN_DIO3              3   //- RF-Modul
//#define PIN_RST_RFM69         4   //- RF-Modul
//#define PIN_DIO5              5   //- RF-Modul
//#define PIN_SCL               6   //- I2C extern
//#define PIN_SDA               7   //- I2C extern
//#define PIN_DIO1              8   //- RF-Modul

//#define GPIO_DIO2              (GPIO_MODER_MODE0 << ( PIN_DIO2 *2 ))      //- IO2 RFM69
//#define GPIO_CURRENT_RF        (GPIO_MODER_MODE0 << ( PIN_CURRENT_RF *2 ))  //- Current RFM69 Modul
//#define GPIO_TX2               (GPIO_MODER_MODE0 << ( PIN_TX2 *2 ))       //- TX-2        
//#define GPIO_RX_2              (GPIO_MODER_MODE0 << ( PIN_RX_2 *2 ))      //- RX-2        
//#define GPIO_DIO0              (GPIO_MODER_MODE0 << ( PIN_DIO0 *2 ))      //- IO0 
//#define GPIO_SCK               (GPIO_MODER_MODE0 << ( PIN_SCK *2 ))       //- RF-Modul
//#define GPIO_MISO              (GPIO_MODER_MODE0 << ( PIN_MISO *2 ))      //- RF-Modul
//#define GPIO_MOSI              (GPIO_MODER_MODE0 << ( PIN_MOSI *2 ))      //- RF-Modul
//#define GPIO_TX1               (GPIO_MODER_MODE0 << ( PIN_TX1 *2 ))       //- RF-Modul
//#define GPIO_RX1               (GPIO_MODER_MODE0 << ( PIN_RX1 *2 ))       //- RF-Modul
//#define GPIO_LED_RED           (GPIO_MODER_MODE0 << ( PIN_LED_RED *2 ))   //- LED Red
//#define GPIO_LED_GREEN         (GPIO_MODER_MODE0 << ( PIN_LED_GREEN *2 )) //- LED Green
//
//
//#define GPIO_CS                (GPIO_MODER_MODE0 << ( PIN_CS *2 ))        //- RF-Modul
//#define GPIO_DIO3              (GPIO_MODER_MODE0 << ( PIN_DIO3 *2 ))      //- RF-Modul
//#define GPIO_RST_RFM69         (GPIO_MODER_MODE0 << ( PIN_RST_RFM69 *2 )) //- RF-Modul
//#define GPIO_DIO5              (GPIO_MODER_MODE0 << ( PIN_DIO5 *2 ))      //- RF-Modul
//#define GPIO_SCL               (GPIO_MODER_MODE0 << ( PIN_SCL *2 ))       //- I2C extern
//#define GPIO_SDA               (GPIO_MODER_MODE0 << ( PIN_SDA *2 ))       //- I2C extern
//#define GPIO_DIO1              (GPIO_MODER_MODE0 << ( PIN_DIO1 *2 ))      //- RF-Modul
//
//
//#define GPIO_OUT(pin)          ( GPIO_MODER_MODE0_0 << (pin*2) )
//#define GPIO_ALT(pin)          ( GPIO_MODER_MODE0_1 << (pin*2) )
//#define GPIO_AN(pin)           ( GPIO_MODER_MODE0 << (pin*2) )
//-----------------------------------------------------------------------------
//-  Pin Mode
//-----------------------------------------------------------------------------
//#define GPIO_MODE_INPUT(__GPIOx__, gpio)               ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(gpio)))
//#define GPIO_MODE_OUTPUT(__GPIOx__, gpio, gpio_out)    ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(gpio)) | ( gpio_out))
//#define GPIO_MODE_ALTERNATE(__GPIOx__, gpio, gpio_alt) ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(gpio)) | ( gpio_alt))
//#define GPIO_MODE_ANALOG(__GPIOx__, gpio, gpio_an)     ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(gpio)) | ( gpio_an))
//
//
//
//#define PIN_MODE_INPUT(__GPIOx__, pins)     ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(GPIO_MODER_MODE0 << (pins*2))))
//#define PIN_MODE_OUTPUT(__GPIOx__, pins)    ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(GPIO_MODER_MODE0 << (pins*2))) | ( GPIO_MODER_MODE0_0 << (pins*2)))
//#define PIN_MODE_ALTERNATE(__GPIOx__, pin)  ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(GPIO_MODER_MODE0 << (pin*2))) | ( GPIO_MODER_MODE0_1 << (pin*2)))
//#define PIN_MODE_ANALOG(__GPIOx__, pin)     ( __GPIOx__->MODER = (__GPIOx__->MODER & ~(GPIO_MODER_MODE0 << (pin*2))) | ( GPIO_MODER_MODE0 << (pin*2)))
//
//#define PIN_NO_PULL(__GPIOx__, pin)         ( __GPIOx__->PUPDR = (__GPIOx__->PUPDR & ~(GPIO_PUPDR_PUPD0 << (pin*2))))
//#define PIN_PULLUP(__GPIOx__, pin)          ( __GPIOx__->PUPDR = (__GPIOx__->PUPDR & ~(GPIO_PUPDR_PUPD0 << (pin*2))) | ( GPIO_PUPDR_PUPD0_0 << (pin*2)))
//#define PIN_PULLDOWN(__GPIOx__, pin)        ( __GPIOx__->PUPDR = (__GPIOx__->PUPDR & ~(GPIO_PUPDR_PUPD0 << (pin*2))) | ( GPIO_PUPDR_PUPD0_1 << (pin*2)))
//
//#define PIN_LOW_SPEED(__GPIOx__, pin)       ( __GPIOx__->OSPEEDR = (__GPIOx__->OSPEEDR & ~(GPIO_OSPEEDER_OSPEED0 << (pin*2))))
//#define PIN_MEDIUM_SPEED(__GPIOx__, pin)    ( __GPIOx__->OSPEEDR = (__GPIOx__->OSPEEDR & ~(GPIO_OSPEEDER_OSPEED0 << (pin*2))) | ( GPIO_OSPEEDER_OSPEED0_0 << (pin*2)))
//#define PIN_HIGH_SPEED(__GPIOx__, pin)      ( __GPIOx__->OSPEEDR = (__GPIOx__->OSPEEDR & ~(GPIO_OSPEEDER_OSPEED0 << (pin*2))) | ( GPIO_OSPEEDER_OSPEED0_1 << (pin*2)))
//#define PIN_VERY_HIGH_SPEED(__GPIOx__, pin) ( __GPIOx__->OSPEEDR = (__GPIOx__->OSPEEDR & ~(GPIO_OSPEEDER_OSPEED0 << (pin*2))) | ( GPIO_OSPEEDER_OSPEED0 << (pin*2)))
//
//#define PIN_ALTERNATE_LOW_AF0( __GPIOx__, pin)   ( __GPIOx__->AFR[0] = (__GPIOx__->AFR[0] & ~(0xF << (pin*4))))
//#define PIN_ALTERNATE_LOW_AF1( __GPIOx__, pin)   ( __GPIOx__->AFR[0] = (__GPIOx__->AFR[0] & ~(0xF << (pin*4))) | ( 0x1 << (pin*4)))
//#define PIN_ALTERNATE_LOW_AF2( __GPIOx__, pin)   ( __GPIOx__->AFR[0] = (__GPIOx__->AFR[0] & ~(0xF << (pin*4))) | ( 0x2 << (pin*4)))
//#define PIN_ALTERNATE_LOW_AF3( __GPIOx__, pin)   ( __GPIOx__->AFR[0] = (__GPIOx__->AFR[0] & ~(0xF << (pin*4))) | ( 0x3 << (pin*4)))
//#define PIN_ALTERNATE_LOW_AF4( __GPIOx__, pin)   ( __GPIOx__->AFR[0] = (__GPIOx__->AFR[0] & ~(0xF << (pin*4))) | ( 0x4 << (pin*4)))
//#define PIN_ALTERNATE_LOW_AF5( __GPIOx__, pin)   ( __GPIOx__->AFR[0] = (__GPIOx__->AFR[0] & ~(0xF << (pin*4))) | ( 0x5 << (pin*4)))
//#define PIN_ALTERNATE_LOW_AF6( __GPIOx__, pin)   ( __GPIOx__->AFR[0] = (__GPIOx__->AFR[0] & ~(0xF << (pin*4))) | ( 0x6 << (pin*4)))
//
//#define PIN_ALTERNATE_HIGH_AF0( __GPIOx__, pin)   ( __GPIOx__->AFR[1] = (__GPIOx__->AFR[1] & ~(0xF << (pin*4))))
//#define PIN_ALTERNATE_HIGH_AF1( __GPIOx__, pin)   ( __GPIOx__->AFR[1] = (__GPIOx__->AFR[1] & ~(0xF << (pin*4))) | ( 0x1 << (pin*4)))
//#define PIN_ALTERNATE_HIGH_AF2( __GPIOx__, pin)   ( __GPIOx__->AFR[1] = (__GPIOx__->AFR[1] & ~(0xF << (pin*4))) | ( 0x2 << (pin*4)))
//#define PIN_ALTERNATE_HIGH_AF3( __GPIOx__, pin)   ( __GPIOx__->AFR[1] = (__GPIOx__->AFR[1] & ~(0xF << (pin*4))) | ( 0x3 << (pin*4)))
//#define PIN_ALTERNATE_HIGH_AF4( __GPIOx__, pin)   ( __GPIOx__->AFR[1] = (__GPIOx__->AFR[1] & ~(0xF << (pin*4))) | ( 0x4 << (pin*4)))
//#define PIN_ALTERNATE_HIGH_AF5( __GPIOx__, pin)   ( __GPIOx__->AFR[1] = (__GPIOx__->AFR[1] & ~(0xF << (pin*4))) | ( 0x5 << (pin*4)))
//#define PIN_ALTERNATE_HIGH_AF6( __GPIOx__, pin)   ( __GPIOx__->AFR[1] = (__GPIOx__->AFR[1] & ~(0xF << (pin*4))) | ( 0x6 << (pin*4)))
//-----------------------------------------------------------------------------
//-  Pin Source for Alternate
//-----------------------------------------------------------------------------

#define B_GPIO5_SOURCE             GPIO_PinSource5   //- RF-Modul
#define B_GPIO3_SOURCE             GPIO_PinSource3   //- RF-Modul
#define B_GPIO2_SOURCE             GPIO_PinSource0   //- RF-Modul
#define B_GPIO1_SOURCE             GPIO_PinSource8   //- RF-Modul
#define B_GPIO0_SOURCE             GPIO_PinSource4   //- RF-Modul

#define RFM69_interruptHandler     EXTI4_15_IRQHandler
//-----------------------------------------------------------------------------
//-  Ports
//-----------------------------------------------------------------------------

#define IF_GPIO5()          (GPIOB->IDR & B_DIO5)
#define IF_GPIO3()          (GPIOB->IDR & B_DIO3)
#define IF_GPIO2()          (GPIOA->IDR & A_DIO2)   
#define IF_GPIO1()          (GPIOB->IDR & B_DIO1)   
#define IF_GPIO0()          (GPIOA->IDR & A_DIO0)   


#define CS_RF_LOW()         (GPIOB->BRR  = B_CS)  //- 
#define CS_RF_HIGH()        (GPIOB->BSRR = B_CS)  //- 


#define LED_RED_LOW()       (GPIOB->BRR  = B_LED_RED)
#define LED_RED_HIGH()      (GPIOB->BSRR = B_LED_RED)
#define LED_RED_TOGGLE()    (GPIOB->ODR ^= B_LED_RED)

#define LED_GREEN_LOW()     (GPIOB->BRR  = B_LED_GREEN)
#define LED_GREEN_HIGH()    (GPIOB->BSRR = B_LED_GREEN)
#define LED_GREEN_TOGGLE()  (GPIOB->ODR ^= B_LED_GREEN)


#define LED_YELLOW_LOW()    (GPIOA->BRR  = A_LED_GELB)
#define LED_YELLOW_HIGH()   (GPIOA->BSRR = A_LED_GELB)
#define LED_YELLOW_TOGGLE() (GPIOA->ODR ^= A_LED_GELB)
//-----------------------------------------------------------------------------
//- clock
//-----------------------------------------------------------------------------

#define  MCK                16000000  //- 16MHz HSI
#define  SYSTICK_CLOCK      (MCK / 1000) //- 1ms Systick
#define  SYSTICK_DISABLE()  (SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk)
#define  SYSTICK_ENABLE()   (SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk)
//-----------------------------------------------------------------------------
//- DAC
//-----------------------------------------------------------------------------

#define DAC_CHANNEL        DAC_Channel_1

//-----------------------------------------------------------------------------
//- SPI
//-----------------------------------------------------------------------------
#define SPI_RF_MODUL       SPI1  //- RF-Modul SPI

//-----------------------------------------------------------------------------
//- I2C
//-----------------------------------------------------------------------------
#define I2C_SENSOR         I2C1  //- immer Slave

//-----------------------------------------------------------------------------
//- USART
//-----------------------------------------------------------------------------

//Vorschlag Mike
#define __DIV(__PCLK, __BAUD)       ((__PCLK*25)/(4*__BAUD))                                               // integerdivider
#define __DIVMANT(__PCLK, __BAUD)   ((__DIV(__PCLK, __BAUD) / 0x64) << 0x04)                               // tmpreg
#define __DIVFRAQ(__PCLK, __BAUD)   (__DIV(__PCLK, __BAUD) - (0x64 * (__DIVMANT(__PCLK, __BAUD) >> 0x04))) //fractionaldivider
#define __USART_BRR(__PCLK, __BAUD) (__DIVMANT(__PCLK, __BAUD)| ((((__DIVFRAQ(__PCLK, __BAUD) * 0x10) + 0x32) / 0x64) & 0x0F))


#define USART_EXTERN          USART1

#define SerialPrint( ... )  (printf(__VA_ARGS__))
//-----------------------------------------------------------------------------
//- macros low power
//-----------------------------------------------------------------------------

//- Enables the Fast WakeUp from Ultra Low Power mode.
//  This bit works in conjunction with ULP bit. 
//  Means, when ULP = 1 and FWU = 1 :VREFINT startup time is ignored when 
//  exiting from low power mode.
#define PWR_EnableFastWakeUp()      PWR->CR |= PWR_CR_FWU
#define PWR_DisableFastWakeUp()     PWR->CR &= (uint32_t)~((uint32_t)PWR_CR_FWU)
#define PWR_EnableUltraLowPower()   PWR->CR |= PWR_CR_ULP
#define PWR_DisableUltraLowPower()  PWR->CR &= (uint32_t)~((uint32_t)PWR_CR_ULP)

//- Low power run mode can only be entered when VCORE is in range 2.
//  In addition, the dynamic voltage scaling must not be used when Low
//  power run mode is selected. Only Stop and Sleep modes with regulator
//  configured in Low power mode is allowed when Low power run mode is 
//  selected.
//  In Low power run mode, all I/O pins keep the same state as in Run mode.
#define PWR_EnableLowPowerRunMode() (PWR->CR |= PWR_CR_LPSDSR\
                                     PWR->CR |= PWR_CR_LPRUN)
#define PWR_DisableLowPowerRunMode() ( PWR->CR &= (uint32_t)~((uint32_t)PWR_CR_LPRUN)\
                                       PWR->CR &= (uint32_t)~((uint32_t)PWR_CR_LPSDSR))

//- Enable or disable the power down mode during Sleep mode.
//  This function is used to power down the FLASH when the system is in SLEEP LP mode.
//  NewState: new state of the power down mode during sleep mode.

//- Set the SLEEP_PD bit to put Flash in power down mode during sleep mode
#define FLASH_SLEEPPowerDownEnable()    FLASH->ACR |= FLASH_ACR_SLEEP_PD
//- Clear the SLEEP_PD bit in to put Flash in idle mode during sleep mode
#define FLASH_SLEEPPowerDownDisable()   FLASH->ACR &= (uint32_t)(~((uint32_t)FLASH_ACR_SLEEP_PD))

//- Enable or disable the power down mode during RUN mode .
//  NewState: new state of the power down mode during RUN mode.

//- Set the RUN_PD bit in  FLASH_ACR register to put Flash in power down mode
// Unlock the RUN_PD bit
/*
#define RAM_FLASH_RUNPowerDownEnable()   (FLASH->PDKEYR = FLASH_PDKEY1 \
                                         FLASH->PDKEYR = FLASH_PDKEY2 \
                                         FLASH->ACR |= (uint32_t)FLASH_ACR_RUN_PD)
//- Clear the RUN_PD bit in  FLASH_ACR register to put Flash in idle  mode
#define RAM_FLASH_RUNPowerDownDisable() FLASH->ACR &= (uint32_t)(~(uint32_t)FLASH_ACR_RUN_PD)
*/

#define PWR_EnableWakeUpPin1()          PWR->CSR |= PWR_WAKEUP_PIN1
#define PWR_DisableWakeUpPin1()         PWR->CSR &= ~PWR_WAKEUP_PIN1

//- Disable the power voltage detector
#define PWR_DisablePVD()                PWR->CR &= (uint32_t)~((uint32_t)PWR_CR_PVDE)

#define enableGlobalInterrupts()   __set_PRIMASK(0);
#define disableGlobalInterrupts()  __set_PRIMASK(1);


//#define __DIV_LPUART(_PCLK_, _BAUD_)                (((_PCLK_)*256)/((_BAUD_)))

//#define USART_BAUD_19200     (((1048000ul)*256)/((19200)))

//-----------------------------------------------------------------------------
//- USART 1
//-----------------------------------------------------------------------------

/* Definition for USARTx clock resources */
#define USARTx                           USART1
#define USARTx_CLK_ENABLE()              __USART1_CLK_ENABLE();
#define USARTx_RX_GPIO_CLK_ENABLE()      __GPIOB_CLK_ENABLE()
#define USARTx_TX_GPIO_CLK_ENABLE()      __GPIOB_CLK_ENABLE() 

#define USARTx_FORCE_RESET()             __USART1_FORCE_RESET()
#define USARTx_RELEASE_RESET()           __USART1_RELEASE_RESET()

/* Definition for USARTx Pins */
#define USARTx_TX_PIN                    B_TXD
#define USARTx_TX_GPIO_PORT              GPIOB  
#define USARTx_TX_AF                     GPIO_AF0_USART1
#define USARTx_RX_PIN                    B_RXD
#define USARTx_RX_GPIO_PORT              GPIOB 
#define USARTx_RX_AF                     GPIO_AF0_USART1

/* Definition for USARTx's NVIC */
#define USARTx_IRQn                      USART1_IRQn
#define USARTx_IRQHandler                USART1_IRQHandler

//-----------------------------------------------------------------------------
//-
//-----------------------------------------------------------------------------  





#endif // #ifndef HW_H
