/******************************************************************************
*                            Steffen Hofmann                                  *
*                            mcu@shotech.de                                   *
*******************************************************************************
*  $Source:  $
*-----------------------------------------------------------------------------*
*  Datum der 2. Version  :  12.03.2017 20:24:04
*  Geraete Bezeichnung   :  RFM69CW-RF-Modul
*-----------------------------------------------------------------------------*
*- $Id:  $
*-----------------------------------------------------------------------------*
*
*	Headerfile
*	----------
*	main Projekt RFM69CW-RF-Modul
* 
*
******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "hw.h"
#include "RFM69.h"
#include "clock.h"
#include "spi.h"
#include "gpio.h"
#include "usart.h"
#include "led.h"
#include <stdio.h>

//****************************************************************************************************************
//**** IMPORTANT RADIO SETTINGS - YOU MUST CHANGE/CONFIGURE TO MATCH YOUR HARDWARE TRANSCEIVER CONFIGURATION! ****
//****************************************************************************************************************
#define NETWORKID     100  //the same on all nodes that talk to each other
#define RECEIVER      1    //unique ID of the gateway/receiver
#define SENDER        2
#define NODEID        RECEIVER  //change to "SENDER" if this is the sender node (the one with the button)
//Match frequency to the hardware version of the radio on your Moteino (uncomment one):
//#define FREQUENCY     RF69_433MHZ
#define FREQUENCY     RF69_868MHZ
//#define FREQUENCY     RF69_915MHZ
#define ENCRYPTKEY    "sampleEncryptKey" //exactly the same 16 characters/bytes on all nodes!
//#define IS_RFM69HW    //uncomment only for RFM69HW! Remove/comment if you have RFM69W!
//*****************************************************************************************************************************
//#define ENABLE_ATC      //comment out this line to disable AUTO TRANSMISSION CONTROL
#define ATC_RSSI        -75
//*********************************************************************************************
#define SERIAL_BAUD   115200

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
  bool bLedstate = LOW;
  bool bInitStatus;
  u8 bRetryWaitTime = 40;
  u8 bRetries = 2;
  
  SysTick_Config(2000); /* 1ms config */
  SystemClock_Config();
  
   if (getError() != 0)
  {
    while(1) /* endless loop */
    {
    }
  }
   
  SysTick_Config(SYSTICK_CLOCK); /* 1ms config */
  SYSTICK_DISABLE();
  
  Configure_GPIO_LED();
  ConfigureGPIO();
  Configure_GPIO_SPI1();
  Configure_SPI1();
  Configure_GPIO_USART1();
  Configure_USART1();
  
  setLedGreenStatus( LED_GREEN_OFF );
  setLedRedStatus( LED_RED_OFF);
  
  
  bInitStatus = RFM69_initialize(FREQUENCY,NODEID,NETWORKID);
  //- error
  if( bInitStatus == false )
  	{
  	   SerialPrint("Init RFM69CW Error !\n");
  	   while(1);
  	}
  	else
  		{
  			SerialPrint("Init RFM69CW OK \n");
  		}

#ifdef IS_RFM69HW
  RFM69_setHighPower(); //only for RFM69HW!
#endif
  RFM69_encrypt(ENCRYPTKEY);
  
#ifdef ENABLE_ATC
  enableAutoPower(ATC_RSSI);
#endif


  char buff[50];
  sprintf(buff, "\nListening at %d Mhz...", FREQUENCY==RF69_433MHZ ? 433 : FREQUENCY==RF69_868MHZ ? 868 : 915);
  SerialPrint(buff);
  
    
  while (1)
  {
      setTickCount( 120 ); //- 120ms
       while(( getTimeout() != true ) && ( !getButton()));
				  
				  if (getButton())
				  {
				    setButton( false );
				    SerialPrint("Button pressed!");
				    //buttonPressed = false;
				    
				    if( bLedstate == LOW )
				    	{
				    		 setLedGreenStatus( LED_GREEN_ON );
				    		 setLedYellowStatus( LED_YELLOW_OFF );
				    		 bLedstate = HIGH;
				      }
				      else
				      	{
				      		setLedGreenStatus( LED_GREEN_OFF );
				      		setLedYellowStatus( LED_YELLOW_ON );
				      		bLedstate = LOW;
				      	}
				    	  

            //- uint8_t toAddress, const void* buffer, uint8_t bufferSize, uint8_t retries=2, uint8_t retryWaitTime=40); // 40ms roundtrip req for 61byte packets
				    if (RFM69_sendWithRetry(RECEIVER, "Hi", 2, bRetries, bRetryWaitTime)) //target node Id, message as string or byte array, message length
				    {
				      //blink LED 3 times, 40ms between blinks
				      Blink(LED_RED, 40, 3 );
				      if( getLedRedStatus() != LED_RED_OFF )
				      	  setLedRedStatus( LED_RED_OFF );
				    }
				  }
  
		  //check if something was received (could be an interrupt from the radio)
		  if (RFM69_receiveDone())
		  {
		    //print message received to serial
		    //SerialPrint('[');Serial.print(radio.SENDERID);Serial.print("] ");
		    sprintf(buff, "[ %d ] ", senderID );
		    SerialPrint( buff );
		    SerialPrint((char*)data);
		    sprintf(buff, "[RX_RSSI: %d ] ", rssi );
		    SerialPrint( buff );
		    //SerialPrint("   [RX_RSSI:");Serial.print(radio.RSSI);Serial.print("]");
		    SerialPrint("\n");
		    
		    //check if received message is 2 bytes long, and check if the message is specifically "Hi"
		    if (datalen==2 && data[0]=='H' && data[1]=='i')
		    {
		    	  LED_RED_TOGGLE();
//		      if(LEDSTATE==LOW)
//		        LEDSTATE=HIGH;
//		      else LEDSTATE=LOW;
		      //digitalWrite(LED, LEDSTATE);
		      //digitalWrite(RX_TOGGLE_PIN, LEDSTATE);
		    }
		   
		    //check if sender wanted an ACK
		    if (RFM69_ACKRequested())
		    {
		      RFM69_sendACK("",0);
		      SerialPrint(" - ACK sent\n");
		    }
		  }
  
  }
}
